declare type GenericFunction<In, Out> = (data: In) => Out | Promise<Out>;
/**
 * Represents a single step in an event.
 */
export declare class Action<I, O> {
    private executionFn;
    private compensation?;
    private retryFn?;
    private next?;
    private constructor();
    /**
     * Create a new Action
     * @param executionFn Function to execute
     * @param compensation Function to undo this action's changes.
     * @param retryFn Function that indicates if to retry execution or not.
     * @returns A new action with the same input and output as `executionFn`
     */
    static new<I, O>(executionFn: GenericFunction<I, O>, compensation?: (data: O, previousData: I, err: any) => any, retryFn?: GenericFunction<Error, boolean>): Action<I, O>;
    /**
     * Execute this action and subsequent ones recursively.
     * @param data Input data for the execution function
     * @returns If the final action in an event, returns a Promise of the output of `executionFn`. Otherwise, returns the next action's execution.
     */
    execute<N>(data: I): Promise<O | N>;
    private executeNext;
    /**
     * Set the next action to be executed (only once).
     * @param action Next action to be executed. Must have the same input as this action's output.
     * @returns Itself
     */
    setNext(action: Action<O, any>): Action<I, O>;
    /**
     * Function to run if subsequent actions fail.
     * @param compensation Function that receives this action's output, input, and the thrown error.
     * @returns Itself
     */
    compensate(compensation: (data: O, previousData: I, err: Error) => any): Action<I, O>;
    /**
     * Set the function that decides if to retry the execution.
     * @param retryFn Function that recieves the thrown error and returns a boolean.
     * @returns Itself
     */
    retry(retryFn: GenericFunction<Error, boolean>): Action<I, O>;
}
export {};
